import boto3
import click
import configparser
import os

class Challenge(object):

    title = 'fake'
    description = 'fake'
    track_name = ''
    track_id = ''

    def __init__(self, track_id, track_name):
        self.track_name = track_name
        self.track_id = track_id

    def help(self):
        self.instructions()

    def start(self):
        self.instructions()
    
    def instructions(self):
        """ Prints the instructions of a challenge """
        click.echo(click.style("Challenge: " + self.title, fg='blue'))
        click.echo(click.style(self.description, fg='black'))
        command = click.style("awslabs {}".format(self.track_id), bold=True)
        click.echo("When finished type: " + command)

    def fail(self, description):
        """ Prints the FAILED message and returns False """
        click.echo(click.style("FAILED", fg='red'))
        click.echo(click.style(description, fg='black'))
        return False
    
    def success(self, description):
        """ Prints the SUCCESS message and returns True """
        click.echo(click.style("SUCCESS", fg='green'))
        click.echo(click.style(description, fg='black'))
        return True

    def save(self, key, value):
        """ Save a value to the config file of this Track """
        file = '~/.awslabs/config'
        filename = os.path.expanduser(file)
        config = configparser.ConfigParser()
        config.read(filename)
        config.set(self.track_id, key, value)
        with open(filename, 'w') as fp:
            config.write(fp)
    
    def get(self, key):
        """ Get a value from the config file of this Track """
        file = '~/.awslabs/config'
        filename = os.path.expanduser(file)
        config = configparser.ConfigParser()
        config.read(filename)
        return config[self.track_id][key]


from awslabs.challenge import Challenge
import boto3

class MyChallenge(Challenge):

    title = "Create an S3 bucket"
    description = "Create an S3 bucket and upload a readme.txt file with the content 'awslabs'."

    def start(self):

        self.instructions()

    def validate(self):

        client = boto3.client('s3')
        buckets = client.list_buckets()
        object_found = False
        usedbucket = ""

        for bucket in buckets['Buckets']:
            try:
                obj = client.get_object(Bucket=bucket['Name'], Key='readme.txt')
                if "awslabs" in obj['Body'].read().decode('utf-8'):
                    object_found = True
                    usedbucket = bucket["Name"]
                    break
            except:
                pass

        if object_found:
            self.save('bucket', usedbucket)
            return self.success("You created a bucket {}, uploaded a file with the correct content.".format(usedbucket))
        else:
            return self.fail("Cannot find a bucket containing a readme.txt with 'awslabs' in it. It should not contain any spaces, new lines etc.")
        



import click
import importlib
import os
from awslabs.tracks.s3.mytrack import MyTrack

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])

def load_class(full_class_string):
    """
    dynamically load a class from a string
    """

    class_data = full_class_string.split(".")
    module_path = ".".join(class_data[:-1])
    class_str = class_data[-1]

    module = importlib.import_module(module_path)
    # Finally, we retrieve the Class
    return getattr(module, class_str)

@click.command(context_settings=CONTEXT_SETTINGS)
@click.argument('track')
@click.argument('action', default='validate', type=click.Choice(['start', 'validate', 'help', 'stop', 'restart']))
def main(track = '', action = ''):

    try:
        trackClass = load_class('awslabs.tracks.{}.mytrack.MyTrack'.format(track))(track)
    except:
        print('Track {} not found'.format(track))
        exit()
    
    getattr(trackClass, action)()


if __name__ == '__main__':
    main()


import boto3
import importlib
import configparser
import os
import click

def load_class(full_class_string):
    """
    dynamically load a class from a string
    """

    class_data = full_class_string.split(".")
    module_path = ".".join(class_data[:-1])
    class_str = class_data[-1]

    module = importlib.import_module(module_path)
    # Finally, we retrieve the Class
    return getattr(module, class_str)


def challenge_id(var):
    var = str(var)
    return '{0}'.format(var.zfill(2))

def load_challenge(track_id, track_name, challenge_id):
    clazz = load_class('awslabs.tracks.{}.challenges.{}.MyChallenge'.format(track_id, challenge_id))
    return clazz(track_id, track_name)

class Track(object):

    id = ''
    name = ''
    description = ''

    def __init__(self, id):
        ''' test '''
        self.id = id

    def start(self):
        if self.get('current') is '0' or self.get('current') is '':
            click.echo(click.style("#################################", fg='red', bold=True))
            click.echo(click.style("# Track: " + self.name, fg='red', bold=True))
            click.echo(click.style("#################################", fg='red', bold=True))
            click.echo(click.style(self.description, fg='black'))
            challenge = load_challenge(self.id, self.name, '01')
            challenge.start()
            self.save('current', 1)
        else:
            print("Track is in progress, use validate to check your current challenge or stop the Track.")

    def help(self):
        challenge = load_challenge(self.id,
                                   self.name,
                                   challenge_id(self.get('current')))
        challenge.start()

    def validate(self):
        self._go()
    
    def _go(self):
        try:
            challenge = load_challenge(self.id,
                                       self.name,
                                       challenge_id(self.get('current')))
        except:
            click.echo("No next challenge found for {}. Have you finished this track?".format(self.id))
            exit()
        # if validate was OK, then load the next challenge start
        if challenge.validate():
            self.save('current', int(self.get('current'))+1)
            try:
                challenge = load_challenge(self.id,
                                           self.name,
                                           challenge_id(int(self.get('current'))))
                challenge.start()
            except:
                # if there is no next challenge, you're finished!
                click.echo("Congratulations, you finished the track: {}".format(self.name))

    def save(self, key, value):
        file = '~/.awslabs/config'
        filename = os.path.expanduser(file)
        dirname = os.path.dirname(filename)
        if not os.path.exists(dirname):
            os.makedirs(dirname)
        config = configparser.ConfigParser()
        config.read(filename)

        # if section does not exist
        if not config.has_section(self.id):
            config.add_section(self.id)

        # add the key and write the file
        config.set(self.id, key, str(value))
        with open(filename, 'w') as fp:
            config.write(fp)
    
    def get(self, key, default = ''):
        file = '~/.awslabs/config'
        filename = os.path.expanduser(file)
        dirname = os.path.dirname(filename)
        if not os.path.exists(dirname):
            os.makedirs(dirname)
        config = configparser.ConfigParser()
        config.read(filename)
        if self.id in config.sections():
            if key in config[self.id]:
                return config[self.id][key]
            else:
                return default
        else:
            return default
    
    def delete(self):
        file = '~/.awslabs/config'
        filename = os.path.expanduser(file)
        config = configparser.ConfigParser()
        config.read(filename)
        config.remove_section(self.id)
        with open(filename, 'w') as fp:
            config.write(fp)

    def stop(self):
        command = click.style("awslabs {} start".format(self.name), bold=True)
        print("Track {} is now stopped. To restart, type: {}".format(self.name, command))
        self.delete()

    def restart(self):
        click.echo("Progress in track {} is deleted. Restarting...".format(self.name))
        self.delete()
        self.start()



from awslabs.challenge import Challenge
import boto3

class MyChallenge(Challenge):

    title = "Launch an instance"
    description = "Create an instance with type t2.small, based on the latest Amazon Linux 2 AMI."

    def start(self):
        self.instructions()

    def validate(self):
        return self.success("You should fix this and that and bla")



from awslabs.challenge import Challenge
import boto3

class MyChallenge(Challenge):

    title = "A Static Website"
    description = "Now turn the created bucket into a website with an index.html containing Hello World"

    def start(self):
        self.instructions()

    def validate(self):

        client = boto3.client('s3')

        bucket_is_website = False
        response = client.get_bucket_website(
                Bucket=self.get('bucket')
            )
        print(response['IndexDocument']['Suffix'])
        try:
            response = client.get_bucket_website(
                Bucket=self.get('bucket')
            )
            if 'IndexDocument' in response and response['IndexDocument']['Suffix'] == 'index.html':
                bucket_is_website = True
        except:
            pass

        if bucket_is_website:
            return self.success("The bucket is indeed a website. Good job!")
        else:
            return self.fail("The bucket {} is NOT a website and NOT serving an index.html containing hello world. Try again.".format(self.get('bucket')))



# -*- coding: utf-8 -*-


'''setup.py: setuptools control.'''


import re
import setuptools
from os import path

here = path.abspath(path.dirname(__file__))
with open(path.join(here, 'README.rst'), 'r') as f:
    long_description = f.read()

version = "0.0.2"

setuptools.setup(
    name='awslabs',
    packages=setuptools.find_packages(),
    entry_points={
        'console_scripts': ['awslabs = awslabs.awslabs:main']
    },
    version=version,
    description='This cli contains aws labs to try.',
    long_description=long_description,
    include_package_data=True,
    zip_safe=False,
    platforms='any',
    install_requires=['boto3', 'requests', 'configparser', 'click', 'cfn_flip', 'pylint'],
    author='Martijn van Dongen',
    author_email='martijnvandongen@binx.io',
    url='https://github.com/binxio/awslabs',
)